/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package model

import (
	"github.com/apache/plc4x/plc4go/spi/utils"
	"github.com/pkg/errors"
)

// Code generated by code-generation. DO NOT EDIT.

// SerialInterfaceAddress is the corresponding interface of SerialInterfaceAddress
type SerialInterfaceAddress interface {
	utils.LengthAware
	utils.Serializable
	// GetAddress returns Address (property field)
	GetAddress() byte
}

// SerialInterfaceAddressExactly can be used when we want exactly this type and not a type which fulfills SerialInterfaceAddress.
// This is useful for switch cases.
type SerialInterfaceAddressExactly interface {
	SerialInterfaceAddress
	isSerialInterfaceAddress() bool
}

// _SerialInterfaceAddress is the data-structure of this message
type _SerialInterfaceAddress struct {
	Address byte
}

///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/////////////////////// Accessors for property fields.
///////////////////////

func (m *_SerialInterfaceAddress) GetAddress() byte {
	return m.Address
}

///////////////////////
///////////////////////
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////

// NewSerialInterfaceAddress factory function for _SerialInterfaceAddress
func NewSerialInterfaceAddress(address byte) *_SerialInterfaceAddress {
	return &_SerialInterfaceAddress{Address: address}
}

// Deprecated: use the interface for direct cast
func CastSerialInterfaceAddress(structType interface{}) SerialInterfaceAddress {
	if casted, ok := structType.(SerialInterfaceAddress); ok {
		return casted
	}
	if casted, ok := structType.(*SerialInterfaceAddress); ok {
		return *casted
	}
	return nil
}

func (m *_SerialInterfaceAddress) GetTypeName() string {
	return "SerialInterfaceAddress"
}

func (m *_SerialInterfaceAddress) GetLengthInBits() uint16 {
	return m.GetLengthInBitsConditional(false)
}

func (m *_SerialInterfaceAddress) GetLengthInBitsConditional(lastItem bool) uint16 {
	lengthInBits := uint16(0)

	// Simple field (address)
	lengthInBits += 8

	return lengthInBits
}

func (m *_SerialInterfaceAddress) GetLengthInBytes() uint16 {
	return m.GetLengthInBits() / 8
}

func SerialInterfaceAddressParse(readBuffer utils.ReadBuffer) (SerialInterfaceAddress, error) {
	positionAware := readBuffer
	_ = positionAware
	if pullErr := readBuffer.PullContext("SerialInterfaceAddress"); pullErr != nil {
		return nil, errors.Wrap(pullErr, "Error pulling for SerialInterfaceAddress")
	}
	currentPos := positionAware.GetPos()
	_ = currentPos

	// Simple Field (address)
	_address, _addressErr := readBuffer.ReadByte("address")
	if _addressErr != nil {
		return nil, errors.Wrap(_addressErr, "Error parsing 'address' field of SerialInterfaceAddress")
	}
	address := _address

	if closeErr := readBuffer.CloseContext("SerialInterfaceAddress"); closeErr != nil {
		return nil, errors.Wrap(closeErr, "Error closing for SerialInterfaceAddress")
	}

	// Create the instance
	return &_SerialInterfaceAddress{
		Address: address,
	}, nil
}

func (m *_SerialInterfaceAddress) Serialize(writeBuffer utils.WriteBuffer) error {
	positionAware := writeBuffer
	_ = positionAware
	if pushErr := writeBuffer.PushContext("SerialInterfaceAddress"); pushErr != nil {
		return errors.Wrap(pushErr, "Error pushing for SerialInterfaceAddress")
	}

	// Simple Field (address)
	address := byte(m.GetAddress())
	_addressErr := writeBuffer.WriteByte("address", (address))
	if _addressErr != nil {
		return errors.Wrap(_addressErr, "Error serializing 'address' field")
	}

	if popErr := writeBuffer.PopContext("SerialInterfaceAddress"); popErr != nil {
		return errors.Wrap(popErr, "Error popping for SerialInterfaceAddress")
	}
	return nil
}

func (m *_SerialInterfaceAddress) isSerialInterfaceAddress() bool {
	return true
}

func (m *_SerialInterfaceAddress) String() string {
	if m == nil {
		return "<nil>"
	}
	writeBuffer := utils.NewWriteBufferBoxBasedWithOptions(true, true)
	if err := writeBuffer.WriteSerializable(m); err != nil {
		return err.Error()
	}
	return writeBuffer.GetBox().String()
}
